// Copyright (c) 2014, The Shelf REST project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_rest.resource;

import 'package:shelf/shelf.dart';
import 'package:shelf_route/shelf_route.dart' show Routeable, Router;

abstract class Resource<T> implements Routeable {
  String get pathParameterName;

  String get _pathParamExpression => pathParameterName != null ?
      '/{$pathParameterName}' : null;

  Handler get searchHandler => null;
  Handler get findHandler => null;
  Handler get postHandler => null;
  Handler get putHandler => null;
  Handler get deleteHandler => null;

  // TODO: do we want to support query params for other methods?
  Iterable<String> get searchQueryParameters => const [];

  Map<dynamic, Resource> get childResources => const {};

  @override
  void createRoutes(Router router) {
    // TODO: add integration tests that test 3 resources deep

    // TODO: search handler needs query params in the route
    // router.get('{?foo,bar}', searchHandler);
    final searchPath = searchQueryParameters.isNotEmpty ?
        '{?${searchQueryParameters.join(",")}}' : '/';

    if (searchHandler != null) {
      router.get(searchPath, searchHandler);
    }

    if (postHandler != null) {
      router.post('/', postHandler);
    }

    final pathParamRouter = _pathParamExpression != null ?
        router.addChildRouter(_pathParamExpression) : router;

    if (findHandler != null) {
      pathParamRouter.get('/', findHandler);
    }
    if (putHandler != null) {
      pathParamRouter.put('/', putHandler);
    }
    if (deleteHandler != null) {
      pathParamRouter.delete('/', deleteHandler);
    }

    childResources.forEach((path, resource) {
      final childResourceRouter = pathParamRouter.addChildRouter(path);
      resource.createRoutes(childResourceRouter);
    });
  }

}

